﻿using PlugInProgram;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CommonUtil;
using System.IO;
using System.Reflection;

namespace HotPluginApp
{
    public partial class Form1 : Form
    {
        const string PLUGIN_TITLE = "GL-HotPluginApp";

        Dictionary<string, List<UserControlBase>> dicLoadedUCs = new Dictionary<string, List<UserControlBase>>();
        List<UserControlBase> lPlugIn = new List<UserControlBase>();
        Panel pnlUC
        {
            get
            {
                return splitContainer1.Panel2;
            }
        }

        public Form1()
        {
            InitializeComponent();

            this.Text = PLUGIN_TITLE;

            splitContainer1.IsSplitterFixed = true;
            splitContainer1.Dock = DockStyle.Fill;

            tvPlugins.Dock = DockStyle.Fill;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            TSMI_ReloadPlugin_Click(null, null);

            Log("App", "Launch");
        }

        private void TSMI_ReloadPlugin_Click(object sender, EventArgs e)
        {
            if (sender != null)
            {
                if (MessageBox.Show("确定重新加载吗？", "Confirm", MessageBoxButtons.YesNo) == DialogResult.No)
                    return;

                pnlUC.Controls.Clear();
            }

            ThreadHelper.RunInAdditionalThread(new DlgtVoidMethod(LoadPlugIns), null, this);
        }

        private void tvPlugins_AfterSelect(object sender, TreeViewEventArgs e)
        {
            this.Text = PLUGIN_TITLE;

            if (tvPlugins.SelectedNode == null
                && tvPlugins.Nodes.Count > 0)
            {
                tvPlugins.SelectedNode = tvPlugins.Nodes[0];
            }
            if (tvPlugins.SelectedNode == null) return;

            if (tvPlugins.SelectedNode.Parent == null
                && tvPlugins.SelectedNode.Nodes.Count > 0)
            {
                tvPlugins.SelectedNode = tvPlugins.SelectedNode.Nodes[0];
            }

            foreach (var _uc in lPlugIn)
            {
                if (_uc.UCName == tvPlugins.SelectedNode.Text)
                {
                    ThreadHelper.RunInAdditionalThread(new DlgtVoidMethod(()=>
                    {
                        LoadUC(_uc, false);
                    }),
                    null,
                    pnlUC);
                    break;
                }
            }
        }


        #region 加载插件
        /// <summary>
        /// 加载插件
        /// </summary>
        void LoadPlugIns()
        {
            // 整理UI
            tvPlugins.Nodes.Clear();
            lPlugIn.Clear();
            dicLoadedUCs.Clear();

            #region 逐一加载UC
            string[] DllFiles = Directory.GetFiles(LoadPlugInManager.PlugInsDir);
            string dllFile = "";
            for (int f = 0; f < DllFiles.Length; f++)
            {
                dllFile = DllFiles[f];

                FileInfo fi = new FileInfo(dllFile);
                if (!fi.Name.EndsWith(".dll")) continue;

                ThreadHelper.RunInAdditionalThread(new DlgtVoidMethod(() =>
                {
                    // 该部分在另一线程中完成，所以不会卡住当前窗体
                    //foreach (var uc in CreatePluginInstance(fi.FullName, this.GetType()))
                    foreach (var uc in LoadPlugInManager.CreateInstance(fi.FullName, this.GetType()))

                    {
                        if (uc != null)
                        {
                            // 保存到已加载UC字典
                            if (!dicLoadedUCs.ContainsKey(uc.UCName))
                            {
                                dicLoadedUCs.Add(uc.UCName, new List<UserControlBase>());
                                dicLoadedUCs[uc.UCName].Add(uc);
                                lPlugIn.Add(uc);

                                // 这里通知窗体线程，加载到插件树控件中（供用户点击选择相应控件）
                                ThreadHelper.RunInAdditionalThread(new DlgtVoidMethod_withParam((Object obj) =>
                                {
                                    UserControlBase _uc = obj as UserControlBase;

                                    TreeNode _tn_ = null;
                                    foreach (TreeNode n in tvPlugins.Nodes)
                                    {
                                        if (n.Text == _uc.UCTpye)
                                        {
                                            _tn_ = n;
                                            break;
                                        }
                                    }
                                    if (_tn_ == null)
                                    {
                                        _tn_ = new TreeNode(_uc.UCTpye);
                                        tvPlugins.Nodes.Add(_tn_);
                                    }
                                    TreeNode _n_ = new TreeNode(_uc.UCName);
                                    _n_.ToolTipText = _uc.Recommend;
                                    _tn_.Nodes.Add(_n_);

                                    tvPlugins.ExpandAll();

                                    Log("App", "成功加载：" + _uc.UCName);
                                })
                                , uc
                                , new DlgtVoidMethod_withParam(delegate (Object oEx)
                                {
                                    MessageBox.Show((oEx as Exception).Message);
                                })
                                , tvPlugins);
                            }
                        }
                    }
                })
                , new DlgtVoidMethod_withParam(delegate (Object oEx)
                {
                    MessageBox.Show((oEx as Exception).Message);
                }));
            }
            #endregion

        }

        void LoadUC(UserControlBase _uc, bool stayTabIdx)
        {
            #region 注入日志记录功能
            if (_uc.Log == null)
                _uc.Log = new DelgLog(delegate (string info)
                {
                    Log(_uc.UCName, info);
                });
            #endregion

            #region Panel加载
            pnlUC.Controls.Clear();
            ThreadHelper.RunInAdditionalThread(new DlgtVoidMethod_withParam(delegate (Object o)
                {
                    pnlUC.Controls.Add((o as UserControlBase));
                })
                , _uc
                , new DlgtVoidMethod_withParam(delegate (Object oEx)
                {
                    MessageBox.Show((oEx as Exception).Message);
                })
                , pnlUC);
            #endregion

            this.Text = PLUGIN_TITLE + "（" + _uc.UCName + "）";
            if (_uc.UcIcon != null)
            {
                this.BeginInvoke(new DlgtVoidMethod_withParam(delegate (Object obj)
                {
                    this.Icon = obj as Icon;
                })
                , _uc.UcIcon);
            }
        }
        #endregion


        #region 日志
        /// <summary>
        /// 日志记录
        /// </summary>
        /// <param name="moduleName">模块标识</param>
        /// <param name="info">日志内容</param>
        public void Log(string moduleName, string info)
        {
            string _logInfo = DateTime.Now.ToYMD_HMS() + "|" + moduleName + "|" + info + Environment.NewLine;

            rtbLog.AppendText_CrossThread(_logInfo);

            LogToFile(moduleName, info);
        }

        public static void LogToFile(string moduleName, string info)
        {
            string _logInfo = DateTime.Now.ToYMD_HMS() + "|" + moduleName + "|" + info;

            string _logDir = Application.StartupPath + "\\Logs";
            if (!System.IO.Directory.Exists(_logDir)) System.IO.Directory.CreateDirectory(_logDir);
            string _logFile = _logDir + "\\HotPluginApp_" + DateTime.Now.ToString("yyyy-MM") + ".log";
            try
            {
                using (System.IO.StreamWriter file = new System.IO.StreamWriter(_logFile, true))
                {
                    file.WriteLine(_logInfo.Replace("\r\n", "；"));
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine(ex);
            }
        }
        #endregion



        public static readonly string PlugInsDir = DirFile_ext.GetPathWithSlash(System.Environment.CurrentDirectory, true) + "PlugIns\\";

        /// <summary>
        /// 加载PlugIns插件目录下的dll
        /// </summary>
        public static List<UserControlBase> GetPlugIns()
        {
            List<UserControlBase> lUc = new List<UserControlBase>();

            foreach (var dllFile in Directory.GetFiles(PlugInsDir))
            {
                FileInfo fi = new FileInfo(dllFile);
                if (!fi.Name.EndsWith(".dll")) continue;

                foreach (var _uc in CreatePluginInstance(fi.FullName))
                {
                    if (_uc != null)
                    {
                        lUc.Add(_uc);
                    }
                }
            }

            return lUc;
        }

        /// <summary>
        /// 根据全名和路径构造对象
        /// </summary>
        /// <param name="sFilePath">程序集路径</param>
        /// <returns></returns>
        public static List<UserControlBase> CreatePluginInstance(string sFilePath, Type hostType = null)
        {
            List<UserControlBase> lUc = new List<UserControlBase>();
            try
            {
                lUc = CreateInstance(sFilePath, new string[] { "Uc" }, hostType);
            }
            catch (Exception ex)
            {
                Console.WriteLine("CreateInstance: " + ex.Message);
            }

            return lUc;
        }

        /// <summary>
        /// 反射创建实例
        /// </summary>
        /// <param name="sFilePath"></param>
        /// <param name="typeFeature"></param>
        /// <param name="hostType"></param>
        /// <param name="dynamicLoad"></param>
        /// <returns></returns>
        public static List<UserControlBase> CreateInstance(string sFilePath, string[] typeFeature, Type hostType = null, bool dynamicLoad = true)
        {
            var lUc = new List<UserControlBase>();
            Assembly assemblyObj = null;

            if (!dynamicLoad)
            {
                #region 方法一：直接从DLL路径加载
                assemblyObj = Assembly.LoadFrom(sFilePath);
                #endregion
            }
            else
            {
                #region 方法二：先把DLL加载到内存，再从内存中加载（可在程序运行时动态更新dll文件，比借助AppDomain方便多了！）
                using (FileStream fs = new FileStream(sFilePath, FileMode.Open, FileAccess.Read))
                {
                    using (BinaryReader br = new BinaryReader(fs))
                    {
                        byte[] bFile = br.ReadBytes((int)fs.Length);
                        br.Close();
                        fs.Close();
                        assemblyObj = Assembly.Load(bFile);
                    }
                }
                #endregion
            }

            if (assemblyObj != null)
            {
                #region 读取dll内的所有类，生成实例（这样可省去提供 命名空间 的步骤）
                // 程序集（命名空间）中的各种类
                foreach (Type type in assemblyObj.GetTypes())
                {
                    try
                    {
                        if (type.ToString().Contains("<>")) continue;
                        if (typeFeature != null)
                        {
                            bool invalidInstance = true;
                            foreach (var tf in typeFeature)
                            {
                                if (type.ToString().Contains(tf))
                                {
                                    invalidInstance = false;
                                    break;
                                }
                            }
                            if (invalidInstance) continue;
                        }

                        var uc = (UserControlBase)assemblyObj.CreateInstance(type.ToString()); //反射创建 
                        lUc.Add(uc);

                        if (hostType != null)
                        {
                            AssemblyInfoHelper aih = new AssemblyInfoHelper(hostType);
                        }
                    }
                    catch (InvalidCastException icex)
                    {
                        Console.WriteLine(icex);
                    }
                    catch (Exception ex)
                    {
                        throw new Exception("Create " + sFilePath + "(" + type.ToString() + ") occur " + ex.GetType().Name + ":\r\n" + ex.Message + (ex.InnerException != null ? "(" + ex.InnerException.Message + ")" : ""));
                    }
                }
                #endregion
            }

            return lUc;
        }

        private void 关于AToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Author: Ken Woo\r\nEmail: suzerain2005@163.com", "Enjoy & Goodluck");
        }
    }


    public class AssemblyInfoHelper
    {
        Type m_Type;
        public AssemblyInfoHelper(Type type)
        {
            this.m_Type = type;
            Assembly assembly = Assembly.GetAssembly(type);
            VersionInfo = assembly.GetCustomAttributes(typeof(AssemblyFileVersionAttribute), false)[0] as AssemblyFileVersionAttribute;
            CompanyInfo = assembly.GetCustomAttributes(typeof(AssemblyCompanyAttribute), false)[0] as AssemblyCompanyAttribute;
            ProductInfo = assembly.GetCustomAttributes(typeof(AssemblyProductAttribute), false)[0] as AssemblyProductAttribute;
            TitleInfo = assembly.GetCustomAttributes(typeof(AssemblyTitleAttribute), false)[0] as AssemblyTitleAttribute;
            CopyrightInfo = assembly.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false)[0] as AssemblyCopyrightAttribute;
            DescriptionInfo = assembly.GetCustomAttributes(typeof(AssemblyDescriptionAttribute), false)[0] as AssemblyDescriptionAttribute;
        }
        /// <summary>
        /// 版本信息
        /// </summary>
        public AssemblyFileVersionAttribute VersionInfo
        {
            get;
            private set;
        }
        /// <summary>
        /// 公司信息
        /// </summary>
        public AssemblyCompanyAttribute CompanyInfo
        {
            get;
            private set;
        }
        /// <summary>
        /// 产品信息
        /// </summary>
        public AssemblyProductAttribute ProductInfo
        {
            get;
            private set;
        }
        /// <summary>
        /// 标题信息
        /// </summary>
        public AssemblyTitleAttribute TitleInfo
        {
            get;
            private set;
        }
        /// <summary>
        /// 版权信息
        /// </summary>
        public AssemblyCopyrightAttribute CopyrightInfo
        {
            get;
            private set;
        }
        /// <summary>
        /// 描述信息
        /// </summary>
        public AssemblyDescriptionAttribute DescriptionInfo
        {
            get;
            private set;
        }

    }

}
